function [ A ] = blkHank( a )
%BLKHANK Creates a block-Hankel matrix from the input block-vector.
%
%   H = blkHank(a) creates a block-Hankel matrix from the blocks in a. It
%   is assumed that the blocks are square of size m-times-m, where m is the
%   width of the matrix a. Considering a to be a block-vector, a is assumed
%   to have an odd number of elements, a = [a_0, ..., a_2k]^T, and the
%   output is
%        ___                   ___
%       | a_0,  a_1,  ... , a_k   |
%   H = | a_1,  a_2,  ... , a_k+1 |
%       |   :    :            :   |
%       | a_k, a_k+1, ... , a_2k  |
%        ---                   ---

[l,k] = size(a);
n_b = l/k;
n = (n_b + 1)/2;

A = zeros(n*k,n*k);
A(1:k,1:n*k) = blkTransp(a(1:n*k,1:k),k);

for i = 1:n-1
    A(k*i+1:k*(i+1), 1:(n-1)*k ) = A(k*(i-1)+1:k*i, k+1:k*n);
    A(k*i+1:k*(i+1), (n-1)*k+1:n*k) = a(n*k+k*(i-1)+1:n*k+k*i,:);
end

end

