function [ A ] = sprBlkDiag( a )
%SPRBLKDIAG creates a sparse, block-diagonal matrix
%
%   A = sprBlkDiag(a) takes a block-column-vector a and returns a sparse
%   block-diagonal matrix A, with the block-elements of a on the diagonal

[l,k] = size(a);
n = l/k;

tmp1 = (1:k).';
tmp2 = (1:k:k*n).';
tmp3 = (1:n*k).';
i = kron(ones(k,1), tmp3);
j = kron(ones(k,1), kron(tmp2, ones(k,1))) + kron(tmp1-1,ones(n*k,1));

A = sparse(i', j(:), a(:));

%% Slower implementation, remove the blkTransp in the beginning to do the same thing for block-row-vectors
% a = blkTransp(a, k);
% tmp1 = (1:k).';
% tmp2 = (0:n-1).';
% i = kron(ones(n*k,1), tmp1) + kron(k*tmp2,ones(k^2,1));
% j = repmat(1:n*k,k,1);
% 
% A = sparse(i', j(:), a(:));

end

