clear all

%% Block-example

% Number of blocks to test for
JkaIter=[800 1600 3200 6400];

% To store data
TBS=[];
stdBS=[];
Tst=[];
stdst=[];


for N=1:length(JkaIter)


n = JkaIter(N);
k = 2;

numLoops = 100;

% To store data
time_backSlash = zeros(numLoops,1);
time_structure = zeros(numLoops,1);
cond_num = zeros(numLoops,1);
max_rel_diff = zeros(numLoops,1);

% Parameter to generate the matrices
lambda=1e-2;

currentTime = clock;
disp(strcat('Start at ', num2str(currentTime(4)), ':', num2str(currentTime(5)), ':', num2str(currentTime(6)), ':'))

for i = 1:numLoops
    disp(['Iteration number ' num2str(i)])

    % Creating matrices which we know are positive definite (see the paper)
    P=randn(n,1)'.^2+1;
    P=[P fliplr(P(2:end))];
    Q=randn(n,1)'.^2+1;
    Q=[Q fliplr(Q(2:end))];

    V=[P./Q.^2; 1./Q; 1./P+lambda./P.^2];
    rst=ifft(V')';
    
    % Measure time in the matrix creating
tic 
    T = [toeplitz(rst(1,1:n)) toeplitz(rst(2,1:n)); toeplitz(rst(2,1:n)) toeplitz(rst(3,1:n))];
    T = T(kron(ones(n,1), [0;n])+kron((1:n)',[1;1]),kron(ones(n,1), [0;n])+kron((1:n)',[1;1]));
    H = [hankel(rst(1,1:n), rst(1,n:2*n-1)) hankel(rst(2,1:n), rst(2,n:2*n-1));...
        hankel(rst(2,1:n), rst(2,n:2*n-1)) hankel(rst(3,1:n), rst(3,n:2*n-1)) ];
    H = H(kron(ones(n,1), [0;n])+kron((1:n)',[1;1]),kron(ones(n,1), [0;n])+kron((1:n)',[1;1]));
    
    t = T(:, 1:k);
    h = [H(:, 1:k); H(end-k+1:end, k+1:end).'];
    
    A = T+H;

    % Removing the intermediate storage of the block-Toeplitz and the
    % block-Hankel matrices
    clear H;
    clear T;

    % Random rhs
    c = rand(n*k,1);
toc

    % Solving the system with \
    tic
    trueSol = A\c;
    time_backSlash(i) = toc;
    
    % Removing the matrix since not needed anymore
    clear A; 
    
    % Solving the system with the "fast" solver
    tic
    compSol = thSolver(t, h, c);
    time_structure(i) = toc;
    
    % To check that the solution is the same
    %cond_num(i) = cond(A);
    max_rel_diff(i) = norm(trueSol - compSol)/norm(trueSol);
    
    currentTime = clock;
    disp(strcat('Iteration finished ', num2str(currentTime(4)), ':', num2str(currentTime(5)), ':', num2str(currentTime(6)), ':'))
end

%Checking how many is solved to different accuracy
NumOk=[length(find(max_rel_diff <= 1e-1)), length(find(max_rel_diff <= 1e-3)), length(find(max_rel_diff <= 1e-5)), length(find(max_rel_diff <= 1e-7))];

disp(strcat('Number of problems solved:', num2str(length(find(max_rel_diff <= 1e-3)))))
disp(strcat('Number of problems solved:', num2str(length(find(max_rel_diff <= 1e-5)))))
disp(strcat('Number of problems solved:', num2str(length(find(max_rel_diff <= 1e-7)))))

% Computing mean and std of solution times
TBS(N)=mean(time_backSlash)
%disp(strcat('Average for \:', num2str(mean(time_backSlash))))
stdBS(N)=std(time_backSlash)
%disp(strcat('Std for \:', num2str(std(time_backSlash))) )

Tst(N)=mean(time_structure)
%disp(strcat('Average for structured:', num2str(mean(time_structure))))
stdst(N)=std(time_structure)
%disp(strcat('Std for structured:', num2str(std(time_structure))) )


%save(['TimesAndMeanfortestThsolver' date num2str(N)], 'stdst', 'Tst', 'TBS', 'stdBS', 'NumOk', 'time_backSlash', 'time_structure', 'JkaIter')

end
