%% Init
clear all
close all

%% Set up
% Parameter
N = 10000;

% Number of moments to match in the different problems

nCases = 100;

% To store data
TBS=[];
stdBS=[];
Tst=[];
stdst=[];

% Running the problems of different sizes
for testfall=1:length(nCases)

n = nCases(testfall);

% Number of problems of each size
numIter = 100;

% To store data
normDiff = zeros(numIter,1);
ex_time = zeros(numIter,1);
iterCount = zeros(numIter,1);

normDiff_blk = zeros(numIter,1);
ex_time_blk = zeros(numIter,1);
iterCount_blk = zeros(numIter,1);

% Running the problems for a given size
for i = 1:numIter
    disp(strcat('Iteration number: ', num2str(i)))

    % Time to set up problem
tic

    % Generate test problem 
    P_true=randn(N,1)'.^2/10+cos(4*pi*(1:N)/N)+3;
    P_true=[P_true + fliplr(P_true)].';
 
    Q_true=randn(N,1)'.^2/30+cos(6*pi*(1:N)/N)+3;
    Q_true=[Q_true + fliplr(Q_true)].';
    
%     plot(1:length(P_true),P_true./Q_true)
    
    spectrum_true = P_true./Q_true;
    c_true = ifft(spectrum_true);
    c_true = real(c_true(1:n+1));
    
    m_true = ifft(log(spectrum_true));
    m_true = real(m_true(2:n+1));
    
toc
    
    % Parameter
    lambda = 1e-2;

    
    % Ordinary solver
    tic
    [q_estim,p_estim,nd,iterCount(i)] = newtonSolver_crce(c_true, m_true, lambda, N);
    ex_time(i) = toc;

    % Postprocessing of solution
    P_estim = real(fft([p_estim(1); 1/2*p_estim(2:end); zeros(N-2*n-1, 1); 1/2*flipud(p_estim(2:end))]));
    Q_estim = real(fft([q_estim(1); 1/2*q_estim(2:end); zeros(N-2*n-1, 1); 1/2*flipud(q_estim(2:end))]));
    spectrum_estim = P_estim./Q_estim;
    
    c_estim = ifft(spectrum_estim);
    diff_c = c_true - c_estim(1:n+1);
    m_estim = ifft( log(spectrum_estim) );
    m_estim = m_estim(2:n+1);
    eps_m = ifft(lambda * 1 ./ P_estim );
    eps_m = eps_m(2:n+1);
    diff_m = m_true - m_estim + eps_m;
    normDiff(i) = norm([diff_c;diff_m],2);
    
    % Fast solver
    tic
    [q_estim_blk,p_estim_blk,nd_blk,iterCount_blk(i)] = newtonSolver_crce_fast(c_true, m_true, lambda, N);
    ex_time_blk(i) = toc;

    % Postprocessing of solution 
    P_estim_blk = real(fft([p_estim_blk(1); 1/2*p_estim_blk(2:end); zeros(N-2*n-1, 1); 1/2*flipud(p_estim_blk(2:end))]));
    Q_estim_blk = real(fft([q_estim_blk(1); 1/2*q_estim_blk(2:end); zeros(N-2*n-1, 1); 1/2*flipud(q_estim_blk(2:end))]));
    spectrum_estim_blk = P_estim_blk./Q_estim_blk;
    
    c_estim_blk = ifft(spectrum_estim_blk);
    diff_c_blk = c_true - c_estim_blk(1:n+1);
    m_estim_blk = ifft( log(spectrum_estim_blk) );
    m_estim_blk = m_estim_blk(2:n+1);
    eps_m_blk = ifft(lambda * 1 ./ P_estim_blk );
    eps_m_blk = eps_m_blk(2:n+1);
    diff_m_blk = m_true - m_estim_blk + eps_m_blk;
    normDiff_blk(i) = norm([diff_c_blk;diff_m_blk],2);

end


disp('Results for \')
avgTime = mean(ex_time);
stdTime = std(ex_time);
solvedProblems = length( find(normDiff < 1e-3 ) )

TBS(testfall)=avgTime
stdBS(testfall)=stdTime

disp('Results for blk-solver')
avgTime_str = mean(ex_time_blk);
stdTime_str = std(ex_time_blk);
solvedProblems_str = length( find(normDiff_blk < 1e-3 ) )

Tst(testfall)=avgTime_str
stdst(testfall)=stdTime_str

%save(['Times_and_mean_for_Newton_' date '_' num2str(testfall)], 'stdst', 'Tst', 'TBS', 'stdBS', 'solvedProblems','solvedProblems_str', 'ex_time','ex_time_blk', 'nCases')

end

